#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>

#include "arg.h"
#include "util.h"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s [-h] [-n num] [-s seed] "
	        "weight1 [weight2 ...]\n", argv0);
}

int
main(int argc, char *argv[])
{
	int i, ret, s = 0, N;
	long j, seed, *counts = NULL, n = 1;
	double val = 0.0, weightsum = 0.0, *weights = NULL;
	struct timeval t1;

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	ARGBEGIN {
	case 'h':
		usage();
		break;
	case 'n':
		n = atol(EARGF(usage()));
		break;
	case 's':
		s = 1;
		seed = atol(EARGF(usage()));
		break;
	default:
		usage();
	} ARGEND;

	if (argc < 1)
		usage();

	N = argc;
	if (!(weights = calloc(N, sizeof(double))) ||
	    !(counts = calloc(N, sizeof(long))))
		err(1, "calloc");

	for (i = 0; i < N; i++) {
		counts[i] = 0;
		weights[i] = atof(argv[i]);
		if (weights[i] <= 0.0)
			errx(1, "weight %d is not positive (%g)", i, weights[i]);
		if (weights[i] > 1.0)
			errx(1, "weight %d is greater than 1 (%g)", i, weights[i]);
	}

	for (i = 0; i < N; i++)
		weightsum += weights[i];
	if (fabs(weightsum - 1.0) > 1e-3)
		errx(1, "weights do not sum to 1 (%g)", weightsum);

	if (s)
		srand48(seed);
	else {
		gettimeofday(&t1, NULL);
		srand48(t1.tv_sec * t1.tv_usec); /* Caveat: identical seed for each ms */
	}

	for (j = 0; j < n; j++) {
		val = drand48();
		weightsum = 0.0;
		for (i = 0; i < N; i++) {
			weightsum += weights[i];
			if (val <= weightsum) {
				counts[i]++;
				break;
			}
		}
	}

	for (i = 0; i < N; i++) {
		printf("%ld", counts[i]);
		if (i < N - 1)
			putchar('\t');
		else
			putchar('\n');
	}

	free(counts);
	free(weights);

	return 0;
}
