#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>
#include <math.h>

#include "arg.h"
#include "util.h"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s [-b] [-e] [-f fmtstr] [-h] [-l] [-n num] [-s] "
	        "[[min_val] max_val]\n", argv0);
}

int
main(int argc, char *argv[])
{
	int i, ret, n = 10, logrange = 0, openstart = 0, openend = 0,
		reportdx = 0;
	double minv = 0.0, maxv = 1.0, dx;
	char fmtstr[PATH_MAX] = "%g\n";

	if (pledge("stdio", NULL) == -1)
		err(2, "pledge");

	ARGBEGIN {
	case 'b':
		openstart = 1;
		break;
	case 'e':
		openend = 1;
		break;
	case 'f':
		ret = snprintf(fmtstr, sizeof(fmtstr), "%s", EARGF(usage()));
		if (ret < 0 || (size_t)ret >= sizeof(fmtstr))
			errx(1, "%s: could not write fmtstr", __func__);
		break;
	case 'h':
		usage();
		break;
	case 'l':
		logrange = 1;
		break;
	case 'n':
		n = atoi(EARGF(usage()));
		break;
	case 's':
		reportdx = 1;
		break;
	default:
		usage();
	} ARGEND;

	if (argc > 2)
		usage();
	else if (argc == 2) {
		minv = atof(argv[0]);
		maxv = atof(argv[1]);
	} else if (argc == 1)
		maxv = atof(argv[0]);

	dx = (maxv - minv) / (n - 1 + openend + openstart);
	if (reportdx) {
		printf(fmtstr, dx);
		return 0;
	}
	for (i = 0 + openstart; i < n + openstart; i++) {
		if (logrange)
			printf(fmtstr, pow(10, minv + i * dx));
		else
			printf(fmtstr, minv + i * dx);
	}

	return 0;
}
