#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <limits.h>
#include <string.h>

#include "arg.h"
#include "util.h"

#define VALUESTR "@VAL@"

char *argv0;

static void
usage(void)
{
	errx(1, "usage: %s [-h] [-n maxiter] cmd min_val max_val\n"
		"where cmd must contain the string '" VALUESTR "'", argv0);
}

static int
launch(char *cmd, char *cmd0, double val)
{
	char *c;

	if ((c = strstr(cmd0, VALUESTR)) == NULL)
		errx(1, VALUESTR " not found in cmd");
	if (strlcpy(cmd, cmd0, PATH_MAX) >= PATH_MAX)
		err(1, "cmd too long");
	sprintf(cmd + (c - cmd0), "%g%s >/dev/null", val, c + strnlen(VALUESTR, sizeof(cmd)));
	if (system(cmd)) {
		fprintf(stderr, "%g\n", val);
		return 1;
	} else
		printf("%g\n", val);

	return 0;
}

static void
binary_search(char *cmd, char *cmd0, double minv, double maxv, int maxiter)
{
	int minfail, maxfail;
	int iter = 0;
	double val;

	minfail = launch(cmd, cmd0, minv);
	maxfail = launch(cmd, cmd0, maxv);
	if (minfail && maxfail)
		errx(2, "both min_val and max_val runs errored");
	else if (!minfail && !maxfail)
		errx(3, "both min_val and max_val ran successfully");

	while (minv <= maxv && iter < maxiter) {
		val = minv + 0.5 * (maxv - minv);
		if (launch(cmd, cmd0, val)) {
			if (maxfail) {
				maxv = val;
				maxfail = 1;
			} else {
				minv = val;
			}
		} else {
			if (maxfail) {
				minv = val;
			} else {
				maxv = val;
				maxfail = 0;
			}
		}
		iter++;
	}
}

int
main(int argc, char *argv[])
{
	int maxiter = 10;
	double minv, maxv;
	char cmd0[PATH_MAX] = "", cmd[PATH_MAX] = "";

	ARGBEGIN {
	case 'h':
		usage();
		break;
	case 'n':
		maxiter = atoi(EARGF(usage()));
		if (maxiter < 1)
			errx(1, "maxiter (-n) must be positive");
		break;
	default:
		usage();
	} ARGEND;

	if (argc == 3) {
		if (strlcpy(cmd0, argv[0], sizeof(cmd0)) >= sizeof(cmd0))
			err(1, "cmd too long");
		minv = atof(argv[1]);
		maxv = atof(argv[2]);
		if (minv >= maxv)
			errx(1, "min_val must be smaller than max_val");
		binary_search(cmd, cmd0, minv, maxv, maxiter);
	} else
		usage();

	return 0;
}
